@@ -1,3 +1,4 @@
+import logging
 from typing import Optional, Text, List, Any
 
 from pybeerxml.fermentable import Fermentable
@@ -8,9 +9,11 @@ from pybeerxml.yeast import Yeast
 from pybeerxml.style import Style
 from pybeerxml.water import Water
 from pybeerxml.equipment import Equipment
-from pybeerxml.utils import cast_to_bool
+from pybeerxml.utils import cast_to_bool, gravity_to_plato
 
-# pylint: disable=too-many-instance-attributes
+logger = logging.getLogger(__name__)
+
+# pylint: disable=too-many-instance-attributes, too-many-statements, too-many-public-methods
 class Recipe:
     def __init__(self):
         self.name: Optional[Text] = None
@@ -45,14 +48,21 @@ class Recipe:
         self.keg_priming_factor: Optional[float] = None
 
         # Recipe extension fields
-        self.est_og = None
-        self.est_fg = None
-        self.est_color = None
-        self.ibu_method = None
-        self.est_abv = None
-        self.actual_efficiency = None
-        self.calories = None
-        self.carbonation_used = None
+        self.est_og: Optional[float] = None
+        self.est_fg: Optional[float] = None
+        self.est_color: Optional[float] = None
+        self.ibu_method: Optional[Text] = None
+        self.est_abv: Optional[float] = None
+        self.actual_efficiency: Optional[float] = None
+        self.calories: Optional[float] = None
+        self.carbonation_used: Optional[Text] = None
+
+        # Values from the recipe, which are calculated as a fallback
+        self._abv = None
+        self._og = None
+        self._fg = None
+        self._ibu = None
+        self._color = None
 
         self.style: Optional[Style] = None
         self.hops: List[Hop] = []
@@ -65,43 +75,110 @@ class Recipe:
 
     @property
     def abv(self):
-        return ((1.05 * (self.og - self.fg)) / self.fg) / 0.79 * 100.0
+
+        if self._abv is not None:
+            return self._abv
+
+        logger.debug("The value for ABV has been calculated from OG and FG")
+        return self.abv_calculated
 
     @abv.setter
-    def set_abv(self, value):
+    def abv(self, value):
+        self._abv = value
+
+    @property
+    def abv_calculated(self):
+        return (
+            ((1.05 * (self.og_calculated - self.fg_calculated)) / self.fg_calculated)
+            / 0.79
+            * 100.0
+        )
+
+    @abv_calculated.setter
+    def abv_calculated(self, value):
         pass
 
     # Gravity degrees plato approximations
     @property
     def og_plato(self):
-        og = self.og
-        return (-463.37) + (668.72 * og) - (205.35 * (og * og))
+        return gravity_to_plato(self.og)
+
+    @og_plato.setter
+    def og_plato(self, value):
+        pass
+
+    @property
+    def og_calculated_plato(self):
+        return gravity_to_plato(self.og_calculated)
+
+    @og_calculated_plato.setter
+    def og_calculated_plato(self, value):
+        pass
 
     @property
     def fg_plato(self):
-        fg = self.fg
-        return (-463.37) + (668.72 * fg) - (205.35 * (fg * fg))
+        return gravity_to_plato(self.fg)
+
+    @fg_plato.setter
+    def fg_plato(self, value):
+        pass
+
+    @property
+    def fg_calculated_plato(self):
+        return gravity_to_plato(self.fg_calculated)
+
+    @fg_calculated_plato.setter
+    def fg_calculated_plato(self, value):
+        pass
 
     @property
     def ibu(self):
 
+        if self._ibu is not None:
+            return self._ibu
+
+        logger.debug(
+            "The value for IBU has been calculated from the hop bill using Tinseth's formula"
+        )
+        return self.ibu_calculated
+
+    @ibu.setter
+    def ibu(self, value):
+        self._ibu = value
+
+    @property
+    def ibu_calculated(self):
+
         ibu_method = "tinseth"
         _ibu = 0.0
 
         for hop in self.hops:
             if hop.alpha and hop.use.lower() == "boil":
-                _ibu += hop.bitterness(ibu_method, self.og, self.batch_size)
+                _ibu += hop.bitterness(ibu_method, self.og_calculated, self.batch_size)
 
         return _ibu
 
-    @ibu.setter
-    def set_ibu(self, value):
+    @ibu_calculated.setter
+    def ibu_calculated(self, value):
         pass
 
     # pylint: disable=invalid-name
     @property
     def og(self):
 
+        if self._og is not None:
+            return self._og
+
+        logger.debug("The value for OG has been calculated from the mashing steps")
+        return self.og_calculated
+
+    @og.setter
+    def og(self, value):
+        self._og = value
+
+    @property
+    def og_calculated(self):
+
         _og = 1.0
         steep_efficiency = 50
         mash_efficiency = 75
@@ -123,14 +200,27 @@ class Recipe:
 
         return _og
 
-    @og.setter
-    def set_og(self, value):
+    @og_calculated.setter
+    def og_calculated(self, value):
         pass
 
     # pylint: disable=invalid-name
     @property
     def fg(self):
 
+        if self._fg is not None:
+            return self._fg
+
+        logger.debug("The value for FG has been calculated from OG and yeast")
+        return self.fg_calculated
+
+    @fg.setter
+    def fg(self, value):
+        self._fg = value
+
+    @property
+    def fg_calculated(self):
+
         _fg = 0
         attenuation = 0
 
@@ -142,16 +232,31 @@ class Recipe:
         if attenuation == 0:
             attenuation = 75.0
 
-        _fg = self.og - ((self.og - 1.0) * attenuation / 100.0)
+        _fg = self.og_calculated - ((self.og_calculated - 1.0) * attenuation / 100.0)
 
         return _fg
 
-    @fg.setter
-    def set_fg(self, value):
+    @fg_calculated.setter
+    def fg_calculated(self, value):
         pass
 
     @property
     def color(self):
+
+        if self._color is not None:
+            return self._color
+
+        logger.debug(
+            "The value for color has been calculated from fermentables using the Morey Equation"
+        )
+        return self.color_calculated
+
+    @color.setter
+    def color(self, value):
+        self._color = value
+
+    @property
+    def color_calculated(self):
         # Formula source: http://brewwiki.com/index.php/Estimating_Color
         mcu = 0.0
         for fermentable in self.fermentables:
@@ -160,8 +265,8 @@ class Recipe:
                 mcu += fermentable.amount * fermentable.color * 8.3454 / self.batch_size
         return 1.4922 * (mcu ** 0.6859)
 
-    @color.setter
-    def set_color(self, value):
+    @color_calculated.setter
+    def color_calculated(self, value):
         pass
 
     @property
@@ -169,5 +274,5 @@ class Recipe:
         return self._forced_carbonation
 
     @forced_carbonation.setter
-    def forced_carbonation(self, value: Any) -> bool:
+    def forced_carbonation(self, value: Any):
         self._forced_carbonation = cast_to_bool(value)
